unit uSimpleDemo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Ruler, RERuler;

type
  TfrmSimpleDemo = class(TForm)
    rulHDemo: TRERuler;
    rulVDemo: TVRuler;
    rchDemo: TRichEdit;
    btnOpen: TButton;
    btnSaveAs: TButton;
    grpDisplayUnits: TRadioGroup;
    grpScale: TRadioGroup;
    grpDrawIndents: TRadioGroup;
    grpBiDiModeRuler: TRadioGroup;
    chkAdjustableMargins: TCheckBox;
    chkAdvancedTabs: TCheckBox;
    chkFlat: TCheckBox;
    chkKeepWithinMargins: TCheckBox;
    chkShowDefaultTabStops: TCheckBox;
    chkShowHints: TCheckBox;
    chkShowIndents: TCheckBox;
    chkShowLastPosition: TCheckBox;
    chkSnapToRuler: TCheckBox;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    Label1: TLabel;
    TrackBar1: TTrackBar;
    Label2: TLabel;
    TrackBar2: TTrackBar;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnSaveAsClick(Sender: TObject);
    procedure grpDisplayUnitsClick(Sender: TObject);
    procedure grpScaleClick(Sender: TObject);
    procedure grpDrawIndentsClick(Sender: TObject);
    procedure grpBiDiModeRulerClick(Sender: TObject);
    procedure chkAdjustableMarginsClick(Sender: TObject);
    procedure chkAdvancedTabsClick(Sender: TObject);
    procedure chkFlatClick(Sender: TObject);
    procedure chkKeepWithinMarginsClick(Sender: TObject);
    procedure chkShowDefaultTabStopsClick(Sender: TObject);
    procedure chkShowHintsClick(Sender: TObject);
    procedure chkShowIndentsClick(Sender: TObject);
    procedure chkShowLastPositionClick(Sender: TObject);
    procedure chkSnapToRulerClick(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure TrackBar2Change(Sender: TObject);
  private
    { Private declarations }
    procedure Bullets(const B: Boolean);
    procedure Clear;
    procedure AddHeader(const S: string);
    procedure AddNormal(const S: string);
    procedure AddSpacer;
  public
    { Public declarations }
  end;

var
  frmSimpleDemo: TfrmSimpleDemo;

implementation

{$R *.DFM}

// ****************************************************************************
// *          The next three events are required to update TRERuler           *
// ****************************************************************************

procedure TfrmSimpleDemo.FormCreate(Sender: TObject);
begin
  Application.ShowHint := True;
  rchDemo.Lines.LoadFromFile('demo.wri');

  rulHDemo.UpdateRulerIndents;
end;

procedure TfrmSimpleDemo.FormPaint(Sender: TObject);
begin
  rulHDemo.SetEditRect;
end;

procedure TfrmSimpleDemo.FormResize(Sender: TObject);
begin
  rulHDemo.SetEditRect;
  rulHDemo.UpdateRulerIndents;
end;


// ****************************************************************************
// *                        Standard Open/Save handling                       *
// ****************************************************************************

procedure TfrmSimpleDemo.btnOpenClick(Sender: TObject);
begin
  dlgOpen.InitialDir := ExtractFilePath(Application.ExeName);
  if dlgOpen.Execute then
    rchDemo.Lines.LoadFromFile(dlgOpen.Filename);
end;

procedure TfrmSimpleDemo.btnSaveAsClick(Sender: TObject);
begin
  if dlgSave.Execute then
    rchDemo.Lines.SaveToFile(dlgSave.Filename);
end;

// ****************************************************************************
// *             Helper routines to write text out to the RichEdit            *
// ****************************************************************************

procedure TfrmSimpleDemo.Bullets(const B: Boolean);
begin
  with rchDemo do
  begin
    if B then
      Paragraph.Numbering := nsBullet
    else
      Paragraph.Numbering := nsNone;
  end;
end;

procedure TfrmSimpleDemo.Clear;
begin
  rchDemo.Clear;
  rchDemo.Paragraph.RightIndent := 0;
end;

procedure TfrmSimpleDemo.AddHeader(const S: string);
begin
  with rchDemo do
  begin
    SelStart  := Length(Text);
    SelLength := 0;
    SelText   := #13#10;
    Paragraph.Alignment := taLeftJustify;
    Paragraph.TabCount  := 0;
    SelAttributes.Style := [fsBold, fsUnderline];
    SelAttributes.Size  := 12;
    SelAttributes.Color := clMaroon;
    SelText := S + #13#10;
  end;
end;

procedure TfrmSimpleDemo.AddNormal(const S: string);
begin
  with rchDemo do
  begin
    Paragraph.Alignment := taLeftJustify;
    SelAttributes.Style := [];
    SelAttributes.Size  := 10;
    SelAttributes.Color := clBlack;
    SelText := S + #13#10;
  end;
end;

procedure TfrmSimpleDemo.AddSpacer;
begin
  with rchDemo do
  begin
    SelStart  := Length(Text);
    SelLength := 0;
    SelText := #13#10;
  end;
end;


// ****************************************************************************
// *             Handle the options which can be set by this demo.            *
// ****************************************************************************

procedure TfrmSimpleDemo.grpDisplayUnitsClick(Sender: TObject);
begin
  case (Sender as TRadioGroup).ItemIndex of
    0: rulHDemo.UnitsDisplay := ruCentimeters;
    1: rulHDemo.UnitsDisplay := ruInches;
    2: rulHDemo.UnitsDisplay := ruMillimeters;
    3: rulHDemo.UnitsDisplay := ruPicas;
    4: rulHDemo.UnitsDisplay := ruPixels;
    5: rulHDemo.UnitsDisplay := ruPoints;
  end;
  rulVDemo.UnitsDisplay := rulHDemo.UnitsDisplay;
end;

procedure TfrmSimpleDemo.grpScaleClick(Sender: TObject);
begin
  with rulHDemo do
    if (Sender as TRadioGroup).ItemIndex = 1 then
      Options := Options + [roScaleRelativeToMargin]
    else
      Options := Options - [roScaleRelativeToMargin];
  with rulVDemo do
    if (Sender as TRadioGroup).ItemIndex = 1 then
      Options := Options + [roScaleRelativeToMargin]
    else
      Options := Options - [roScaleRelativeToMargin];
end;

procedure TfrmSimpleDemo.grpDrawIndentsClick(Sender: TObject);
begin
  with rulHDemo.IndentSettings do
    if (Sender as TRadioGroup).ItemIndex = 1 then
      Options := Options + [ioExtraShadow]
    else
      Options := Options - [ioExtraShadow];
end;

procedure TfrmSimpleDemo.grpBiDiModeRulerClick(Sender: TObject);
begin
  case (Sender as TRadioGroup).ItemIndex of
    0: rulHDemo.BiDiModeRuler := bmLeftToRight;
    1: rulHDemo.BiDiModeRuler := bmRightToLeft;
    2: rulHDemo.BiDiModeRuler := bmUseBiDiMode;
  end;
end;

procedure TfrmSimpleDemo.chkAdjustableMarginsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Adjustable margins');
  AddSpacer;
  AddNormal('Gives the user the possibility to change the margins of the '+
            'non-printable area of the paper by dragging the margin markers. '+
            'You will see a crSizeWE cursor when you are at the margin marker.');
  AddNormal('For the vertical ruler the default drag cursor is crSizeNS. '+
            'Although you can drag the Top and BottomMargings there is no code '+
            'in this demo which actually changes the Top and BottomMargins '+
            'of the RichEdit.');
  with rulHDemo.MarginSettings do
    if chkAdjustableMargins.Checked then
      Options := Options + [moAdjustable]
    else
      Options := Options - [moAdjustable];
  with rulVDemo.MarginSettings do
    if chkAdjustableMargins.Checked then
      Options := Options + [moAdjustable]
    else
      Options := Options - [moAdjustable];
end;

procedure TfrmSimpleDemo.chkAdvancedTabsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Advanced tabs');
  AddSpacer;
  AddNormal('This option is only useful for RichEdit V2 and later like '+
            'TRichEdit98 or TrxRichEd. When this option is turned on, a '+
            'popup menu is shown when right-clicking a tab. In this popup '+
            'menu you can select different types of tab settings like; ' );
  Bullets(True);
  AddNormal('Left align (the default)');
  AddNormal('Center align');
  AddNormal('Right align');
  AddNormal('Decimal align');
  AddNormal('Word Bar align');
  Bullets(False);
  AddSpacer;
  AddNormal('Notes:');
  Bullets(True);
  AddNormal('if you don''t see a tab you can add one by clicking on the '+
            'empty area within the ruler.');
  AddNormal('you can remove tabs by dragging them outside the ruler area.');
  Bullets(False);
  with rulHDemo.TabSettings do
    if chkAdvancedTabs.Checked then
      Options := Options + [toAdvancedTabs]
    else
      Options := Options - [toAdvancedTabs];
end;

procedure TfrmSimpleDemo.chkFlatClick(Sender: TObject);
begin
  rulHDemo.Flat := chkFlat.Checked;
  rulVDemo.Flat := chkFlat.Checked;
end;

procedure TfrmSimpleDemo.chkKeepWithinMarginsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Keep indents within margins');
  AddSpacer;
  AddNormal('You can use this option to force the indents to stay within the '+
            'margins. This one is set to True by default because this was '+
            '(roughly) the default behavior of the previous versions.');
  AddNormal('BTW turning this option off is not very useful when using a '+
            'standard TRichEdit.');
  with rulHDemo.IndentSettings do
    if chkKeepWithinMargins.Checked then
      Options := Options + [ioKeepWithinMargins]
    else
      Options := Options - [ioKeepWithinMargins];
end;

procedure TfrmSimpleDemo.chkShowDefaultTabStopsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Default tab stops');
  AddSpacer;
  AddNormal('When turned on little markers at the bottom of the ruler shows '+
            'you the default tab stops of the RichEdit.');
  with rulHDemo.TabSettings do
    if chkShowDefaultTabStops.Checked then
      Options := Options + [toShowDefaultTabs]
    else
      Options := Options - [toShowDefaultTabs];
end;

procedure TfrmSimpleDemo.chkShowHintsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Show hints');
  AddSpacer;
  AddNormal('When turned on hints are displayed when the mouse hovers over '+
            'the indents and/or tabs.');
  rulHDemo.ShowHint := chkShowHints.Checked;
end;

procedure TfrmSimpleDemo.chkShowIndentsClick(Sender: TObject);
begin
  Clear;
  AddHeader('Show Indents');
  AddSpacer;
  AddNormal('Turns on/off the drawing routines for indent markers.');
  with rulHDemo.IndentSettings do
    if chkShowIndents.Checked then
      Options := Options + DefaultIndentOptions
    else
      Options := Options - DefaultIndentOptions;
end;

procedure TfrmSimpleDemo.chkShowLastPositionClick(Sender: TObject);
begin
  Clear;
  AddHeader('Show last position');
  AddSpacer;
  AddNormal('When enabled the last position where the Indent/Tab was is drawn '+
            'dimmed when you drag an Indent/Tab around.');
  if chkShowLastPosition.Checked then
    rulHDemo.Options := rulHDemo.Options + [roItemsShowLastPos]
  else
    rulHDemo.Options := rulHDemo.Options - [roItemsShowLastPos];
end;

procedure TfrmSimpleDemo.chkSnapToRulerClick(Sender: TObject);
begin
  Clear;
  AddHeader('Snap to ruler');
  AddSpacer;
  AddNormal('You can use this option to force the changes made to margins, '+
            'indents and tabs to line-up at fixed intervals. '+
            'When the UnitsDisplay property is set to ruCentimeter the grid '+
            'step is 0.25 cm when it is set to ruInches the steps are 1/16". '+
            'These steps start to count from the LeftMargin.');
  AddNormal('When the above option is turned off, you can drag the margins, '+
            'indents and tabs freely to any position you like.');
  AddNormal('Although in this demo there is only one checkbox to turn on/off '+
            '"Snap to ruler" you can set/clear these options separately for '+
            'the margins, indents and/or tabs.');
  with rulHDemo.MarginSettings do
    if chkSnapToRuler.Checked then
      Options := Options + [moSnapToRuler]
    else
      Options := Options - [moSnapToRuler];
  with rulHDemo.IndentSettings do
    if chkSnapToRuler.Checked then
      Options := Options + [ioSnapToRuler]
    else
      Options := Options - [ioSnapToRuler];
  with rulHDemo.TabSettings do
    if chkSnapToRuler.Checked then
      Options := Options + [toSnapToRuler]
    else
      Options := Options - [toSnapToRuler];

  with rulVDemo.MarginSettings do
    if chkSnapToRuler.Checked then
      Options := Options + [moSnapToRuler]
    else
      Options := Options - [moSnapToRuler];
end;

procedure TfrmSimpleDemo.TrackBar1Change(Sender: TObject);
begin
  rulHDemo.Inset := TrackBar1.Position;
  rulVDemo.Inset := TrackBar1.Position;
  Label1.Caption := 'Inset ('+ IntToStr(TrackBar1.Position) + ')';
end;

procedure TfrmSimpleDemo.TrackBar2Change(Sender: TObject);
begin
  rulHDemo.Zoom := TrackBar2.Position;
  rulVDemo.Zoom := TrackBar2.Position;
  Label2.Caption := 'Zoom ('+ IntToStr(TrackBar2.Position) + '%)';
end;

end.
